@@ -1,5 +1,5 @@
 package Selenium::Remote::Commands;
-$Selenium::Remote::Commands::VERSION = '0.2250'; # TRIAL
+$Selenium::Remote::Commands::VERSION = '0.2350'; # TRIAL
 # ABSTRACT: Implement commands for Selenium::Remote::Driver
 
 use Moo;
@@ -347,7 +347,7 @@ has '_cmds' => (
             'uploadFile' => {
                 'method'             => 'POST',
                 'url'                => 'session/:sessionId/file',
-                'no_content_success' => 1
+                'no_content_success' => 0
             },
             'getLocalStorageItem' => {
                 'method'             => 'GET',
@@ -358,7 +358,42 @@ has '_cmds' => (
                 'method'             => 'DELETE',
                 'url'                => '/session/:sessionId/local_storage/key/:key',
                 'no_content_success' => 1
-            }
+            },
+            'cacheStatus' => {
+                'method'             => 'GET',
+                'url'                => 'session/:sessionId/application_cache/status',
+                'no_content_success' => 0
+            },
+            'setGeolocation' => {
+                'method'             => 'POST',
+                'url'                => 'session/:sessionId/location',
+                'no_content_success' => 1
+            },
+            'getGeolocation'   => {
+                'method'             => 'GET',
+                'url'                => 'session/:sessionId/location',
+                'no_content_success' => 0
+            },
+            'getLog' => {
+                'method'             => 'POST',
+                'url'                => 'session/:sessionId/log',
+                'no_content_success' => 0
+            },
+            'getLogTypes'   => {
+                'method'             => 'GET',
+                'url'                => 'session/:sessionId/log/types',
+                'no_content_success' => 0
+            },
+            'setOrientation' => {
+                'method'             => 'POST',
+                'url'                => 'session/:sessionId/orientation',
+                'no_content_success' => 1
+            },
+            'getOrientation'   => {
+                'method'             => 'GET',
+                'url'                => 'session/:sessionId/orientation',
+                'no_content_success' => 0
+            },
 
             # /session/:sessionId/local_storage
             # /session/:sessionId/local_storage/key/:key
@@ -426,7 +461,7 @@ Selenium::Remote::Commands - Implement commands for Selenium::Remote::Driver
 
 =head1 VERSION
 
-version 0.2250
+version 0.2350
 
 =head1 SEE ALSO
 
